import 'package:cl_appbase/api.dart';
import 'package:cl_datahub_common/common.dart';

class ApiResource<TData extends TransferObject> {
  final String endpointPostfix;
  final DTOFactory<TData> factory;
  final ApiBase api;

  const ApiResource._(this.api, this.endpointPostfix, this.factory);
}

class ListApiResource<TData extends TransferObject, TId>
    extends ApiResource<TData> {
  const ListApiResource(
      ApiBase api, String endpointPostfix, DTOFactory<TData> factory)
      : super._(api, endpointPostfix, factory);

  Future<List<TData>> getList(int offset, int limit,
      [Map<String, dynamic>? params]) async {
    final response = await api.getList<TData>(endpointPostfix,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> getElement(TId id, [Map<String, dynamic>? params]) async {
    final response = await api.getObject('$endpointPostfix/$id',
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> postElement(TData element,
      [Map<String, dynamic>? params]) async {
    final response = await api.postObject(endpointPostfix, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> patchElement(TData element,
      [Map<String, dynamic>? params]) async {
    final url =
        element is IntIdTransferObject || element is StringIdTransferObject
            ? '$endpointPostfix/${(element as dynamic).id}'
            : endpointPostfix;
    final response = await api.patchObject(url, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  //TODO delete
}

class SingleObjectApiResource<TData extends TransferObject>
    extends ApiResource<TData> {
  const SingleObjectApiResource(
      ApiBase api, String endpointPostfix, DTOFactory<TData> factory)
      : super._(api, endpointPostfix, factory);

  Future<TData> getElement([Map<String, dynamic>? params]) async {
    final response = await api.getObject('$endpointPostfix',
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> postElement(TData element,
      [Map<String, dynamic>? params]) async {
    final response = await api.postObject(endpointPostfix, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> patchElement(TData element,
      [Map<String, dynamic>? params]) async {
    final response = await api.patchObject(endpointPostfix, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }
}
